<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
 <xsl:output method="text"
	    encoding="ISO-8859-1"/>
<xsl:template match="package" mode="classpath">lib/<xsl:value-of select="@name"/>.<xsl:value-of select="@version"/>.jar$(PATH_SEPARATOR)</xsl:template>		
<xsl:template match="package" mode="lib">lib/<xsl:value-of select="@name"/>.<xsl:value-of select="@version"/>.jar </xsl:template>		

<xsl:template match="/">
CYGWIN=$(shell test -x /usr/bin/cygpath &amp;&amp; echo true || echo false)
ifeq ($(CYGWIN),true)
	PATH_SEPARATOR=;
else
	PATH_SEPARATOR=:
endif	
	
HASJARPATH=$(shell test -d "$(JARPATH)" &amp;&amp; echo true ||echo false)
ifeq ($(HASJARPATH),false)
	JARPATH=lib
endif

PACKAGE=<xsl:value-of select="/package/@name"/>
VERSION=<xsl:value-of select="/package/@version"/>
MAINCLASS=<xsl:value-of select="/package/@main"/>

all: $(PACKAGE).$(VERSION).jar
	make all.local

all.local::

JAVAC=javac
JAR=jar
JAVA=java
JAVADOC=javadoc -private
DEBUG=-O
CLASSP=<xsl:apply-templates select="/package/require/package" mode="classpath"/>bin
LIBS=<xsl:apply-templates select="/package/require/package" mode="lib"/>
ifeq ($(JARPATH),lib)
DISTLIB=
else
DISTLIB=$(LIBS) lib/junit.jar
endif
JFLAGS=-classpath "${CLASSP}" -d bin -sourcepath src $(DEBUG)
CLASSFILES=$(patsubst src/%.java,bin/%.class,$(shell find src -name '*.java'))
TESTCLASSP=$(CLASSP)$(PATH_SEPARATOR)$(JARPATH)/junit.jar$(PATH_SEPARATOR)tbin
TESTCLASSFILES=$(patsubst test/%.java,tbin/%.class,$(shell find test -name '*.java'))
TESTSUITE=$(patsubst tbin/%.class,%,$(TESTCLASSFILES))

HASLOCAL=$(shell test -f src/Makefile &amp;&amp; echo true || echo false)
ifeq ($(HASLOCAL),true)
include src/Makefile
endif

bin/%.class: src/%.java .init.dirs $(LIBS)
	${JAVAC} ${JFLAGS} $&lt; 

tbin/%.class: test/%.java .init.dirs $(LIBS)
	${JAVAC} -classpath "${TESTCLASSP}" -d tbin -sourcepath test $(DEBUG) $&lt;

%: tbin/%.class
	$(JAVA) -cp "$(TESTCLASSP)" junit.textui.TestRunner $@
	
lib/%: $(JARPATH)/% .init.dirs
	ln -s $&lt; $@

Makefile: jmakefile.xsl src/package.xml
	xsltproc jmakefile.xsl src/package.xml &gt; Makefile
	
.init.dirs: Makefile
	mkdir -p bin lib tbin obj doc ; \
	date > .init.dirs ; \
	make .init.dirs.local
	
.init.dirs.local::

clean:
	rm -rf tbin bin obj *.jar .init.dirs ; \
	make clean.local

clean.local::	

clean.doc:
	rm -rf doc ; \
	make clean.doc.local

clean.doc.local::	

clean.all: clean clean.doc clean.dist

$(PACKAGE).$(VERSION).jar: $(CLASSFILES)
	cd bin &amp;&amp; \
	$(JAR) cvf ../$(PACKAGE).$(VERSION).jar * &amp;&amp; \
	cd .. 

install: $(PACKAGE).$(VERSION).jar
	mkdir -p ${JARPATH} ; \
	cp $(PACKAGE).$(VERSION).jar ${JARPATH} ; \
	make install.local
	
install.local::

run: $(PACKAGE).$(VERSION).jar
	$(JAVA) -cp "$(CLASSP)" $(MAINCLASS) $(RUNARGS) ; \
	make run.local

run.local::	

doc: .init.dirs
	$(JAVADOC) -d doc -classpath "$(CLASSP)" -sourcepath src -subpackages $(PACKAGE) ; \
	make doc.local

doc.local::	
	
test: all $(TESTSUITE)
	make test.local
	
test.local::

$(PACKAGE)-src.$(VERSION).tar.gz: Makefile $(DISTLIB)
	mkdir -p $(PACKAGE)-src.$(VERSION) ; \
	ln -s ../src $(PACKAGE)-src.$(VERSION)/src ; \
	ln -s ../test $(PACKAGE)-src.$(VERSION)/test ; \
	ln -s ../lib $(PACKAGE)-src.$(VERSION)/lib ; \
	ln -s ../Makefile $(PACKAGE)-src.$(VERSION)/Makefile ; \
	ln -s ../jmakefile.xsl $(PACKAGE)-src.$(VERSION)/jmakefile.xsl ; \
	tar cvzhf $(PACKAGE)-src.$(VERSION).tar.gz \
		--exclude .svn --exclude CVS $(PACKAGE)-src.$(VERSION) ; \
	rm -rf $(PACKAGE)-src.$(VERSION)	

dist.src: $(PACKAGE)-src.$(VERSION).tar.gz		
	make dist.local

clean.dist:
	rm -rf $(PACKAGE)-src.$(VERSION).tar.gz

dist.local::

dist.bin: $(PACKAGE).$(VERSION).jar
	echo "Main-Class: "$(MAINCLASS) > Manifest ; \
	echo "Class-Path: "$(LIBS) >> Manifest ; \
	jar uvmf Manifest $(PACKAGE).$(VERSION).jar $(LIBS) ; \
	rm Manifest ; \
	make dist.bin.local

dist.bin.local::	


check.dist: $(PACKAGE)-src.$(VERSION).tar.gz
	tar -xvzf $(PACKAGE)-src.$(VERSION).tar.gz ; \
	cd $(PACKAGE)-src.$(VERSION) ; \
	make JARPATH=lib test ; \
	make check.dist.local ; \
	cd .. ; \
	rm -rf $(PACKAGE)-src.$(VERSION)

check.dist.local::	
	
</xsl:template>	
</xsl:stylesheet>
