/*  $Id: TaskletContext.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.core;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import net.eduvax.util.Logger;

/**
 * Tasklet execution context.
 */
public class TaskletContext {
    // ------------------------------------------------------
    // PUBLIC METHODS
    /**
     * create new tasklet context
     */
    public TaskletContext(ExecRequest request,LocalNode n) {
        execRequest=request;
        node=n;
        out=null;
    }            
    /**
     * @return execute request
     */
    public ExecRequest getExecRequest() {
        return execRequest;     
    }        
    /**
     * @return tasklet context standard output
     */
    public PrintStream getPrintStream() {
        if (out==null) {
            try {
                // PrintStream creation is delayed here to not
                // create an empty unused output file for each tasklet
                // but only for the ones that are requesting for a
                // PrintStream.
                out=new PrintStream(new FileOutputStream(
                        "out-"+execRequest.getTaskId()+".grant"));                
            }
            catch (IOException ex) {
                Logger.getSLogger("net.eduvax.grant").warn(
                      "Can't create tasklet output "+ex.getMessage());    
                out=System.out;    
            }        
        }
        return out;     
    }        
    /**
     * @return node that is running the tasklet.
     */
    public Node getNode() {
        return node;
    }    
    /**
     * Fire runtime event.
	 * Due to actual event transport communication, data must be serializable
	 * as a string (with toString() methods). Registered listener will recieve
	 * data as String class containing this serialization.
	 * @param data data to attach to the event.
     */
    public void sendEvent(Object data) {
        node.sendEvent(TaskletEvent.RUNTIME, 
				new TaskletEvent(execRequest,data));
	}
    // ------------------------------------------------------
    // PRIVATE PROPERTIES
    /** standard output */
    private PrintStream out;    
    /** execute request */
    private ExecRequest execRequest;
    /** node resource for tasklet. */
    private LocalNode node;
}        
