/*  $Id: TaskletEventDispatcher.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.core;

import java.util.Hashtable;
import java.util.Vector;

/**
 * Tasklet event dispatcher.
 * Register and notify tasklet event listener.
 * Dispatcher itself is the main tasklet event listener.
 */
public class TaskletEventDispatcher implements TaskletEventListener {
    // -----------------------------------------------------------
    // PUBLIC METHODS
    /**
     * create new dispatcher.
     * Each node should create only one event dispatcher instance.
     */
    TaskletEventDispatcher() {    
        listeners=new Hashtable();            
    }    
    /**
     * Register an event listener.
     * There is no public deregister methods, all listener are deregistered
     * after canceled and completed events handling.
     * @param er exec request of the tasklet to listening to.
     * @param listener tasklet event listener.
     */ 
    public void register(ExecRequest er,TaskletEventListener listener) {
        String id=er.getTaskId();    
        Vector v;
        synchronized(listeners) {
            v=(Vector)listeners.get(id);           
            if (v==null) {
                v=new Vector();
                listeners.put(id,v);
            }                
            v.addElement(listener);
        }    
    }            
    // ---- TaskletEventListner implementation ----
    public void scheduled(TaskletEvent ev) {
        Vector v=(Vector)listeners.get(ev.getExecRequest().getTaskId());
        if (v!=null) {
            for (int i=0;i<v.size();i++) {
                ((TaskletEventListener)v.elementAt(i)).scheduled(ev);        
            }        
        }                
    }    
    // ---- TaskletEventListner implementation ----
    public void delayed(TaskletEvent ev) {
        Vector v=(Vector)listeners.get(ev.getExecRequest().getTaskId());
        if (v!=null) {
            for (int i=0;i<v.size();i++) {
                ((TaskletEventListener)v.elementAt(i)).cancelled(ev);        
            }        
        }                
    }    
    // ---- TaskletEventListner implementation ----
    public void runtimeEvent(TaskletEvent ev) {
        Vector v=(Vector)listeners.get(ev.getExecRequest().getTaskId());
        if (v!=null) {
            for (int i=0;i<v.size();i++) {
                ((TaskletEventListener)v.elementAt(i)).cancelled(ev);        
            }        
        }                
    }            
    // ---- TaskletEventListner implementation ----
    public void cancelled(TaskletEvent ev) {
        Vector v=(Vector)listeners.get(ev.getExecRequest().getTaskId());
        if (v!=null) {
            for (int i=0;i<v.size();i++) {
                ((TaskletEventListener)v.elementAt(i)).cancelled(ev);        
            }        
        }                
        removeTaskletListeners(ev.getExecRequest());            
    }            
    // ---- TaskletEventListner implementation ----
    public void completed(TaskletEvent ev) {   
        Vector v=(Vector)listeners.get(ev.getExecRequest().getTaskId());
        if (v!=null) {
            for (int i=0;i<v.size();i++) {
                ((TaskletEventListener)v.elementAt(i)).completed(ev);        
            }        
        }                
        removeTaskletListeners(ev.getExecRequest());            
    }    
    // -----------------------------------------------------------
    // PRIVATE METHODS
    /**
     * Remove ALL listeners associated to specified tasklet.
     * @param er exec request (defines the observed tasklet).
     */  
    private void removeTaskletListeners(ExecRequest er) {
        String id=er.getTaskId();    
        Vector v;
        synchronized(listeners) {
            v=(Vector)listeners.get(id);           
            if (v!=null) {
                listeners.remove(id);                    
            }        
        }    
    }            
    // -----------------------------------------------------------
    // PRIVATE PROPERTIES    
    /** 
     * @djin.aggregation - * net.eduvax.grant.core.TaskletEventListener       
     */ 
    private Hashtable listeners;    
}        
