/*  $Id: TaskletJarCache.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.core;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.jar.JarFile;
import java.util.StringTokenizer;

import net.eduvax.util.Logger;
import net.eduvax.util.SLogger;

/**
 *
 */
public class TaskletJarCache {
    public TaskletJarCache() {
    }        
    /**
     *
     */         
    public TaskletJarCache(String prefix) {
        pathPrefix=prefix;            
    }        
    /**
     * get local file path for cached tasklet jar file
     * @param jarURL tasklet jar file URL
     */
    public String getCachedJarPath(String jarURL) {
        return pathPrefix+"/tasklet"+jarURL.hashCode()+".jar";
    }            
    /**
     * get tasklet jar file from the web if not yet present
     * on local disk, then store it on local disk.
     * @param taskletURL tasklet URL.
     * @return stored jar file.
     */         
    public JarFile get(String taskletURL) {
        JarFile res=null;
        StringTokenizer st=new StringTokenizer(taskletURL,"#");
        String jarURL=st.nextToken();
        String fullJarFilePath=getCachedJarPath(jarURL);
        try {
            File jarFile=new File(fullJarFilePath);
            if (jarFile.exists()) {
                res=new JarFile(jarFile);                
            }
            else {
                URL url=new URL(jarURL);
                URLConnection connection=url.openConnection();        
                InputStream in=connection.getInputStream();
                FileOutputStream out=new FileOutputStream(fullJarFilePath);
                int ch;
                do {
                    ch=in.read();
                    if (ch>=0) {
                        out.write(ch);                            
                    }                    
                } while (ch>=0);                        
                in.close();
                out.close();
                logger.info("Downloaded tasklet code from "+jarURL);
                res=new JarFile(jarFile);
            }
        }        
        catch (IOException ioex) {
            logger.error("Can't get tasklet jar file "+jarURL+" : "+ioex);
        }            
        return res;
    }        
    /** cached jar path prefix. */
    private String pathPrefix=".";
	/** logger. */
	static private SLogger logger;
    static {
		logger=Logger.getSLogger("net.eduvax.grant");
    }
}        
