/*  $Id: TestA.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.simul;

import java.io.IOException;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import net.eduvax.grant.core.ExecReport;
import net.eduvax.grant.core.ExecRequest;
import net.eduvax.grant.core.Node;
import net.eduvax.grant.core.NodeClient;
import net.eduvax.grant.core.NodeFactory;
import net.eduvax.util.Trace;

/**
 * 1st test suite : local nodes and jobs got from standard input.
 * @version
 * $Id: TestA.java 238 2006-04-30 20:07:59Z sdevaux $
 * @author Sebastien Devaux
 */
public class TestA extends Test implements NodeFactory, NodeClient {
	/** level 1 node count. */
	public static final int L1COUNT = 5;
	/** level 2 node count. */
	public static final int L2COUNT = 6;
	/** high delay for "low nodes". */
	public static final int DELAY_HIGH = 200;
	/** medium nodes. */
	public static final int DELAY_MEDIUM = 100;
	/** best nodes. */
	public static final int DELAY_LOW = 0;
	/**
	 * init test.
	 * create nodes.
	 */
	public void init() {
        Node n,m;
        Node.registerNodeFactory("TestA",this);
		String nodeName;
		nodes=new Hashtable();
		terminalNodes=new Vector();
		root=createNode("TestA:0:root");
        root.removeNode("TestA:0:root");
		for (int i=0;i<L1COUNT;i++) {
			nodeName="TestA:0:node-"+i;
			n=root.addNode(nodeName);
            n.removeNode(nodeName);
			for (int j=0;j<L2COUNT;j++) {
				int delay=0;
				if (i%2>0) {
					if (j%2>0) {
						delay=DELAY_HIGH;
					}
					else {
						delay=DELAY_MEDIUM;
					}
				} 
				else {
					if (j%2>0) {
						delay=DELAY_MEDIUM;
					}
					else {
						delay=DELAY_LOW;
					}
				}
				nodeName="TestA:"+delay+":node-"+i+"."+j;
				m=n.addNode(nodeName);
				terminalNodes.add(m);
			}
		}
	}
	/**
	 * trace all nodes state.
	 */
	public void traceNodes() {
/*        
		Iterator it=nodes.values().iterator();
		while (it.hasNext()) {
			((Node)it.next()).trace();
		}
*/        
	}
	/**
	 * set test arguments.
	 * Should have only one : node type number.
	 * @param c test arguments
	 */
	public void setArgs(Collection c) {
		Iterator it=c.iterator();
		String unhandled="";
		while (it.hasNext()) {
			String optarg=(String)it.next();
			if ("-ntype".equals(optarg)) {
				nodeType=Integer.parseInt((String)it.next());
				Trace.time("[TestA] setting node type to "+nodeType);
			} 
			else if ("-lcost".equals(optarg)) {
				nodeLoadCost=Integer.parseInt((String)it.next());
				Trace.time("[TestA] setting node load cost to "+nodeLoadCost);
			} 
			else {
				unhandled+=optarg+" ";
			}
		}
		if (!"".equals(unhandled)) {
			Trace.time("[TestA] unhandled params at this level : "+unhandled);
		}
	}
	/**
	 * create a node according type number defined for this test.
	 * All created nodes for one test are same type, one of following :<ul>
	 * <li>Round Robin Node.</li>
	 * <li>Full Random Node.</li>
	 * <li>Ant Test Node.</li></ul>
	 * @param name new node's name.
	 * @return new node.
	 */
	public Node createNode(String name) {
		TestNode n;
        StringTokenizer st=new StringTokenizer(name,":");
        st.nextToken();
        int power=Integer.parseInt(st.nextToken());
		switch(nodeType) {
			case 1:
				n=new TestRoundRobinNode(name,power);
				break;
			case 2:
				n=new TestRandomNode(name,power);
				break;
			default:
				n=new TestNode(name,power);
				break;
		} 
		n.setClient(this);
		n.setLoadCost(nodeLoadCost);
		return n;
	}
	/**
	 * Test A implementation : reads input stream, 2 number per line.
	 * 1st is test tasklet delay, 2nd is pause before next tasklet launch.
	 */
	public void run() {
		try {
			String line;
			int i=0;
			while ((line=input.readLine())!=null) {
				StringTokenizer st=new StringTokenizer(line);
				String cmd=st.nextToken();
				if ("launch".equals(cmd)) {
					int delay=Integer.parseInt(st.nextToken());
					ExecRequest request=new ExecRequest(
                                    "file:///e:/jdev/jars/net.eduvax.grant.0.2.2d.jar#net.eduvax.grant.simul.TestTasklet",""+delay);
					if (nodeType!=3) {
                        routeReport(root.submitRequest(request));
					}
					else {
						// this case try to be the best possible : choose 
						// directly best node at schedule time.
						routeReport(getBestNode().submitRequest(request));
					}
					i++;
					int pause=Integer.parseInt(st.nextToken());
					Thread.sleep(pause);
				}
				else if ("setperf".equals(cmd)) {
					traceNodes();
					String nodeName=st.nextToken();
					int value=Integer.parseInt(st.nextToken());
					TestNode n=(TestNode)nodes.get(nodeName);
					if (n!=null) {
						n.setPerf(value);
						trace("setPerf ["+nodeName+"]: "+value);
					}
				}
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}	
		catch (InterruptedException ex) {
			ex.printStackTrace();
		}	
	}
	/** 
	 * trace tasklet scheduling.
	 * @param request scheduled request.
	 */
	public void routeReport(ExecRequest request) {
		trace(request.getTaskId()+" ["+request.getTaskURL()
					+" "+request.getArgs()
					+"] scheduled : "+request.getRouterPath());
		load++;
		trace("load: "+load);
	}
	/** 
	 * trace tasklet execution.
	 * @param report task completion report
	 */
	public void execReport(ExecReport report) {
		trace(report.getRequest().getTaskId()+" ["
					+report.getRequest().getTaskURL()+" "
					+report.getRequest().getArgs()
					+"] completed by "+report.getRequest().getRouterPath()
					+" : "+report.getCr()+" ["+report.getTime()+"]");
		load--;
		trace("load: "+load);
		if (load==0) {
			System.exit(0);
		}
	}
	/** 
	 * get the lowest loaded terminal node.
	 */
	private Node getBestNode() {
		float bestLoad=0; 
		Node bestNode=null;
		Iterator it=terminalNodes.iterator();
		if (it.hasNext()) {
			do {
				TestNode n=(TestNode)it.next();
				if (bestNode==null||bestLoad>n.getLoadValue()) {
					bestNode=n;
					bestLoad=n.getLoadValue();
				}
			} while (it.hasNext());
		}
		return bestNode;
	}
	/** 
	 * trace  a message.
	 * @param msg message to trace.
	 */
	protected void trace(String msg) {
		Trace.time("[TestA] "+msg);
	}
	/** all nodes. */
	private Hashtable nodes;
	/** terminal nodes. */
	private Vector terminalNodes;
	/** root node. */
	private Node root;
	/** load. */
	private int load=0;
	/** node type. */
	private int nodeType=0;
	/** node load cost. */
	private int nodeLoadCost=0;
}
