/*  $Id: TestNode.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.simul;

import java.io.File;
import java.util.Hashtable;

import net.eduvax.grant.core.ExecRequest;
import net.eduvax.grant.core.ExecReport;
import net.eduvax.grant.core.LocalNode;
import net.eduvax.grant.core.NodeClient;
import net.eduvax.util.DateTime;

/**
 * Evaluation node.
 * This node's goal is to help to evaluate Ant Engine selection. It is intended
 * to use pausing test tasklet and simulates itself cpu load by another pause.
 * This pause delay depends on node virtual capacity and can envolve to simulate
 * real cpu load variations.
 * @version
 * $Id: TestNode.java 238 2006-04-30 20:07:59Z sdevaux $
 * @author Sebastien Devaux
 */
class TestNode extends LocalNode {
	/**
	 * Creates new test node.
	 * @param n new node's name
	 * @param p virtual cpu load. The node will pause p seconds when computing
	 *  	cpu load (@see #load()) i.e. after each tasklet completion.
	 */
	public TestNode(String n,int p) {
		super(n);
		perfIndice=p;
		loadCost=0;
	}
	/**
	 * Changes performance indice value.
	 * This affects pause length in load() method.
	 * @param p new perf indice value.
	 */
	public void setPerf(int p) {
		perfIndice=p;
	}
	/**
	 * Changes load cost.
	 * Delay value added for each running tasklet.
	 * @param c delay value in milliseconds.
	 */
	public void setLoadCost(int c) {
		loadCost=c;
	}
	/**
	 * computes CPU Load.
	 * return virtual cpu load and pause according current simulated load.
	 * @return current load value.
	 */
	public float load() {
		try {
			Thread.sleep((long)getLoadValue());
		}
		catch (InterruptedException ex) {
			ex.printStackTrace();
		}
		return perfIndice;
	}
	/**
	 *
	 */
	public int execReportToClient(ExecReport er) {
		client.execReport(er);
		return 0;
	}
	/**
	 *
	 */
	public void setClient(NodeClient nc) {
		client=nc;
	}	
	/**
	 * Compute "real" load for test node.
	 * Used to computed fake delay and te let simulation know at any time, which
	 * node is best to compare with tested algorithmes choices.
	 */
	public float getLoadValue() {
		return perfIndice*DateTime.ONE_SEC_MS+getTaskCount()*loadCost;
	}
	/**
	 * Do not feedback on schedule.
	 * @param er execute request
	 */
	protected float getFeedback( ExecRequest er) {
		return 0.0f;
	}
    /**
     * Fake rc file loading.
     * Nothing to load with this test node.
     * @param nodeSet not used here
     * @param nodesFile not used here
     */
    protected void loadNodes(Hashtable nodeSet,File nodesFile) {
    }
    /**
     * Fake load check loading.
     * @return true
     */
    protected boolean checkLoad() {
        return true;
    } 
	/** current perf indice. */
	private int perfIndice;
	/** delay to add for each running task. */
	private int loadCost;
	/** */
	NodeClient client;
}
