/*  $Id: Mandelbrot.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tasklet example
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.tasklets.fractal;

import java.util.StringTokenizer;

import net.eduvax.grant.core.AbstractTasklet;
import net.eduvax.grant.core.Node;
import net.eduvax.grant.core.ExecRequest;

import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;


/**
 * Mandelbrot set computation.
 * @version $Id: Mandelbrot.java 238 2006-04-30 20:07:59Z sdevaux $
 * @author Sebastien Devaux
 */
public class Mandelbrot extends AbstractTasklet {
    private int computePoint(double x,double y) {
        double zr=0;    
        double zi=0;
        double r2=0;
        double i2=0;
        int iter=0;
        while (iter<maxIter && r2+i2<=4.0) {
            zi=zr*zi*2;
            zi+=y;
            zr=r2-i2;
            zr+=x;
            r2=zr*zr;
            i2=zi*zi;            
            iter++;
        }        
        return iter;
    }        

    private void computeAll() {
        int w=(int)((xstop-xstart)/step)+1;    
        int h=(int)((ystop-ystart)/step)+1;    
        BufferedImage image=new BufferedImage(w,h,
                        BufferedImage.TYPE_USHORT_565_RGB);            
        int py=0;
        for (double y=ystart;y<ystop;y+=step) {    
            int px=0;    
            for (double x=xstart;x<xstop;x+=step) {
                image.setRGB(px,py,computePoint(x,y));    
                px++;
            }
            py++;        
        }   
        try {
            ImageIO.write(image, "PNG", context.getPrintStream());
        }
        catch (IOException ex) {
            context.getPrintStream().println("IOError : "+ex.getMessage());
            exitCode=1;    
        }                
    }        

	private void schedule(double step) {
		for (double x=-2.0;x<2.0;x+=step) {
			double x1=x+step;
			for (double y=-2.0;y<2.0;y+=step) {
				double y1=y+step;
				Node n=context.getNode();
                ExecRequest er=new ExecRequest(
                    context.getExecRequest().getTaskURL(),
					""+x+" "+x1+" "+y+" "+y1+" "+step/1000.0+" "+65536);
				n.submitRequest(er);
                context.getPrintStream().println("submited "+er.getTaskURL()
                                                    +" "+er.getArgs());
			}
		}
	}

    public void run() {
		String args=context.getExecRequest().getArgs();
        StringTokenizer st=new StringTokenizer(
              context.getExecRequest().getArgs());      
		if (args.startsWith("step")) {
			st.nextToken();
			schedule(Double.parseDouble(st.nextToken()));
		}
		else {
			if (st.hasMoreTokens())  
				xstart=Double.parseDouble(st.nextToken());    
			if (st.hasMoreTokens())  
				xstop=Double.parseDouble(st.nextToken());    
			if (st.hasMoreTokens())  
				ystart=Double.parseDouble(st.nextToken());    
			if (st.hasMoreTokens())  
				ystop=Double.parseDouble(st.nextToken());    
			if (st.hasMoreTokens())  
				step=Double.parseDouble(st.nextToken());    
			if (st.hasMoreTokens())  
				maxIter=Double.parseDouble(st.nextToken());    
			computeAll();
		}
    }        

    private double xstart= -1.0;
    private double xstop =  1.0;
    private double ystart= -1.0;
    private double ystop =  1.0;
    private double step  =  0.01;
    private double maxIter= 512;
}
