/*  $Id: NodeHttpHandler.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.xmlrpc;

import net.eduvax.grant.core.Node;
import net.eduvax.util.Logger;

import org.apache.xmlrpc.XmlRpcServer;
import org.mortbay.http.HttpRequest;
import org.mortbay.http.HttpResponse;
import org.mortbay.http.HttpException;
import org.mortbay.http.handler.AbstractHttpHandler;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

/**
 * Grant http request handler.
 */
public class NodeHttpHandler extends AbstractHttpHandler {
	/**
	 * create new http handler
	 * @param n server node.
	 * @param xNode node's xmlrpc request handler
	 */
    public NodeHttpHandler(Node n,NodeXMLrpcServer xNode) {
		node=n;
        xmlrpcNode=xNode;
        xmlServer = new XmlRpcServer();
        xmlServer.addHandler("grant",xmlrpcNode);
    } 
	//----------------------------------------------------
	// Abstract http handler implementation
	public void handle(String pathInContext,
	                   String pathParams,
					   HttpRequest request,
					   HttpResponse response)
					   throws HttpException, IOException {
        String method=request.getMethod();                               
        if (HttpRequest.__GET.equals(method)) {
            handleGet(pathInContext,pathParams,request,response);     
        }
        else if (HttpRequest.__POST.equals(method)) {
            handlePost(pathInContext,pathParams,request,response);     
        }        
	}
	//----------------------------------------------------
	/**
	 * handle POST http requests.
	 * POST requests are handled by cmlrpc handler.
	 */
	public void handlePost(String pathInContext,
	                   String pathParams,
					   HttpRequest request,
					   HttpResponse response)
					   throws HttpException, IOException {
        byte[] result = xmlServer.execute(request.getInputStream());
        response.setContentType("text/xml");
        response.setContentLength(result.length);
        OutputStream out = response.getOutputStream();
        out.write(result);
        out.close();
	}
	/**
	 * handle GET http requests.
	 * handled URLs :<ul>
	 *	<li>/ : returns node status in xml format.</li>
	 *	<li>/out?tpid=&lt;tasklet process id&gt; : returns tasklet process 
	 *	output.</li>
	 */
	public void handleGet(String pathInContext,
	                   String pathParams,
					   HttpRequest request,
					   HttpResponse response)
					   throws HttpException, IOException {
		if ("/".equals(pathInContext)) {
			sendNodeStatus(request,response);
		}
		else if ("/out".equals(pathInContext)) {
			sendTaskletOutput(request,response);
		}
		else {
			response.setStatus(response.__404_Not_Found);
			request.setHandled(true);					   
		}
		
	}
	/**
	 * send node status in http response.
	 * @param request http request
	 * @param response http response to write into
	 */
	private void sendNodeStatus(HttpRequest request,HttpResponse response) {
		response.setStatus(response.__200_OK);					   
		response.setContentType("text/xml");
        PrintWriter out=new PrintWriter(response.getOutputStream());
        out.print(node.status());
		request.setHandled(true);					   
        out.close();
	}
	/**
	 * send one tasklet output in http response.
	 * @param request http request. A parameter named 'tpid' is expected and
	 *	must contain a tasklet process id.
	 * @param response http response to write into
	 */
	private void sendTaskletOutput(HttpRequest request,HttpResponse response) {
		String tpid=request.getParameter("tpid");
		File file=new File("out-"+tpid+".grant");
		if (file.exists()) {
			response.setStatus(response.__200_OK);
			try {
				OutputStream out=response.getOutputStream();
				InputStream input=new FileInputStream(file);
				int c;
				while ((c=input.read())>=0) {
					out.write((byte)c);
				}
				out.close();
			}
			catch (IOException ex) {
				Logger.getSLogger("net.eduvax.grant").error(
					"Can't send tasklet ["+tpid+"] : "+ex.getMessage());
			}
		}
		else {
			response.setStatus(response.__404_Not_Found);
		}
		request.setHandled(true);					   
	}
	/** xmlrpc server */
    private XmlRpcServer xmlServer;
	/** served node */
	private Node node;
	/** xmlrpc server handler */
	private NodeXMLrpcServer xmlrpcNode;
}
