/*  $Id: NodeXMLrpcServer.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.xmlrpc;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.Properties;

import net.eduvax.grant.core.ExecReport;
import net.eduvax.grant.core.ExecRequest;
import net.eduvax.grant.core.LocalNode;
import net.eduvax.grant.core.Node;
import net.eduvax.grant.core.TaskletEvent;

import net.eduvax.util.AbstractConfigChecker;
import net.eduvax.util.ConfigChecker;
import net.eduvax.util.ConfigHandler;



/**
 * XML RPC Server interface.
 * maps node service to XML-RPC protocole.
 * @version $Id: NodeXMLrpcServer.java 238 2006-04-30 20:07:59Z sdevaux $
 * @author Sebastien Devaux
 */
public class NodeXMLrpcServer implements Runnable {
	/**
	 * node's executeRequest method wrapper.
	 * @param taskId task id.
	 * @param taskURL tasklet URI.
	 * @param args "command line" arguments.
	 * @param routerPath accumulating routing path.
	 * @param status route status
	 * @return xml rpc call result as a Map : should contain en entry
	 *	called "routerPath" that is the full routing path. 
	 */
	public Hashtable submitRequest(String taskId, 
							String taskURL,String args,String routerPath,
							String status) {
		Hashtable result=new Hashtable();
		ExecRequest er=
				new ExecRequest(taskId,taskURL,args,routerPath,status);
		ExecRequest routed=node.submitRequest(er);				
		result.put("routerPath", routed.getRouterPath());
		result.put("status", ""+routed.getRouteStatus());
		return result;
	}
	/**
	 * node's executeReport method wrapper.
	 * @param taskId task id.
	 * @param taskURL tasklet URI.
	 * @param args "command line" arguments.
	 * @param routerPath accumulating routing path.
	 * @param routeStatus routing status
	 * @param cr tasklet execution completion code (exit code).
	 * @param time tasklet execution time (execution duration).
	 * @param cpu cpu power used/available for/while running the tasklet.
	 * @return xml rpc call result as a Map : should contain en entry
	 *	called "rc" that is the tasklet execution completion code. 
	 */
	public Hashtable execReport(String taskId, 
							String taskURL, String args,String routerPath,
							String routeStatus,
							int cr,long time,float cpu) {
		Hashtable result=new Hashtable();
		ExecRequest request=
				new ExecRequest(taskId,taskURL,args,routerPath,routeStatus);
		ExecReport report=new ExecReport(request,cr,time,cpu);
		result.put("rc",
			new Integer(node.execReport(report))
		);
		return result;
	}
	/**
	 * @return node load
	 */
	public Hashtable load() {
		Hashtable result=new Hashtable();
		result.put("load",new Float(node.load()));
		return result;
	}
	/**
	 * @return node status
	 */
	public Hashtable status() {
		Hashtable result=new Hashtable();
		result.put("status",node.status());
		return result;
	}
	/**
	 * handle shutdown request
	 */
	public Hashtable shutdown() {
		Hashtable result=new Hashtable();
		webserver.shutdown();
		node.shutdown();
		return result;
	}
	/**
	 * Adding a node
	 * @param name new node name
	 */
	public Hashtable addNode(String name) {
		node.addNode(name);
		return new Hashtable();
	}
	/**
	 * removing a node
	 * @param name new node name
	 */
	public Hashtable removeNode(String name) {
		node.removeNode(name);
		return new Hashtable();
	}
	/**
	 * handle event
	 * @param eventType event type
	 * @param taskId task id.
	 * @param taskURL tasklet URI.
	 * @param args "command line" arguments.
	 * @param routerPath accumulating routing path.
	 * @param status route status
	 * @param data event data
	 */ 
	public Hashtable handleEvent(int eventType,
					String taskId,String taskURL,
					String args,String routerPath,
					String routeStatus,String data) {
		ExecRequest request=
				new ExecRequest(taskId,taskURL,args,routerPath,routeStatus);
		node.handleEvent(eventType,new TaskletEvent(request,data));
		return new Hashtable();
	}
	/**
	 * create xmlrpc server handler.
	 * The server is listening to the default HTTP port (80)
	 * @param n node to wrap with this new server.
	 */
	public NodeXMLrpcServer(Node n) {
		this(n,DEFAULT_HTTP_PORT);
	}
	/**
	 * create xmlrpc server handler.
	 * The server is listening to specified HTTP port.
	 * @param n node to wrap with this new server.
	 * @param port TCP port number to listen to.
	 */
	public NodeXMLrpcServer(Node n,int port) {
		node=n;
		webserver=new WebServer(port,n,this);
	}
	/**
     * run webserver.
	 */
	public void run() {
		webserver.start();
	}
	/**
	 * create and run a node and xmlrpc server.
	 * @param argv command line arguments. Not used yet.
	 */ 
	public static void main(String[] argv) {
		// sets default arguments
		Properties defaultConfig=new Properties();
        defaultConfig.setProperty("port",""+DEFAULT_HTTP_PORT);
        defaultConfig.setProperty("maxload",""+DEFAULT_MAXLOAD);
		int cmd=CMD_HELP;
		final int CMD_ARG=0;
		String hostname="";
        try {
            hostname=InetAddress.getLocalHost().getHostName();    
        } 
        catch (UnknownHostException ex) {
            hostname="localhost";
        }
        defaultConfig.setProperty("node",hostname);
		// read command line args
		String switches="";
		if (argv.length<=CMD_ARG) {
			cmd=CMD_HELP;
		}
		else if ("start".equals(argv[CMD_ARG])) {
			cmd=CMD_START;
			switches=" node port maxload ";
		}
		else if ("stop".equals(argv[CMD_ARG])) {
			cmd=CMD_STOP;
			switches=" node port ";
		}
		else if ("add".equals(argv[CMD_ARG])) {
			switches=" node port rnode ";
			cmd=CMD_ADD;
		}
		else if ("remove".equals(argv[CMD_ARG])) {
			switches=" node port rnode ";
			cmd=CMD_REMOVE;
		}
		else if ("status".equals(argv[CMD_ARG])) {
			cmd=CMD_STATUS;
			switches=" node port ";
		}
		else if ("submit".equals(argv[CMD_ARG])) {
			switches=" node port tasklet targ ";
			cmd=CMD_SUBMIT;
		}
		else {
			System.out.println("Unknown command :"+argv[CMD_ARG]);
		}
		final String knownSwitches=switches;	
		ConfigChecker checker=new AbstractConfigChecker() {
			public int checkProperty(String name,String value) {
				if (knownSwitches.indexOf(name)>=0) {
					return CC_OK;
				}
				else {
					System.err.println("Unsupported option : -"+name+" "+value);
					return CC_ERR;
				}	
			}
		};
		ConfigHandler configHandler=new ConfigHandler(
			defaultConfig,
			"~/.grant/grant.properties",
			argv,
			checker);
		if (configHandler.load()!=ConfigChecker.CC_OK) {
			cmd=CMD_HELP;
		}
		Properties config=configHandler.get();
		// format some args
		String nodeURL=config.getProperty("node");
		if (nodeURL==null||hostname.equals(nodeURL)) {
	        nodeURL="xmlrpc+http://"+hostname;        
			int port=Integer.parseInt(config.getProperty("port"));
		    if (port!=DEFAULT_HTTP_PORT) {
			    nodeURL=nodeURL+":"+port;
	        }
		}	
		// run selected command.
		switch (cmd) {
			case CMD_START :
				int port=Integer.parseInt(config.getProperty("port"));
                int maxLoad=Integer.parseInt(config.getProperty("maxload"));
				cmdStart(nodeURL,port,maxLoad);
				break;
			case CMD_STOP :
				(new XMLrpcRemoteNode(nodeURL)).shutdown();
				break;
			case CMD_ADD :
				(new XMLrpcRemoteNode(nodeURL))
						.addNode(config.getProperty("rnode"));
				break;
			case CMD_REMOVE :
				(new XMLrpcRemoteNode(nodeURL))
						.removeNode(config.getProperty("rnode"));
				break;
			case CMD_STATUS :
				System.out.println(
						(new XMLrpcRemoteNode(nodeURL)).status());
				break;
            case CMD_SUBMIT :
                cmdSubmit(nodeURL,
						config.getProperty("tasklet"),
						config.getProperty("targ"));
                break;                
			default:
				help();
				break;
		}
	}
	/**
	 *
	 */
	public static void cmdStart(String nodeURL,int port,int maxLoad) {	
		new XMLrpcNodeFactory(); // register xmlrpc node factory
		LocalNode n=new LocalNode(nodeURL);
        n.setMaxLoad(maxLoad);
		NodeXMLrpcServer xmlServer=new NodeXMLrpcServer(n,port);
		xmlServer.run();
	}
    /**
     *
     */
    public static void cmdSubmit(String nodeURL,String tasklet,
                    String taskletArgs) {
        if (tasklet==null) {
            help();
        }
        else {
            ExecRequest er=new ExecRequest(tasklet,taskletArgs);
            (new XMLrpcRemoteNode(nodeURL)).submitRequest(er);
        }        
    }        
	/**
	 *
	 */
	public static void help() {
		System.out.println("Grant xmlrpc node server.");
		System.out.println("Known commands :");
		System.out.println("	start|stop|add|remove|status|submit");
		System.out.println("	start [-port numPort] [-maxload max] : start node server on localhost.");
		System.out.println("	stop [-node nodeURL] : send stop request to server.");
		System.out.println("	add [-node nodeURL] -rnode rnodeURL : add rnodeURL to nodeURL's active nodes list.");
		System.out.println("	remove [-node nodeURL] -rnode rnodeURL : remove rnodeURL frome nodeURL's active nodes list.");
        System.out.println("	submit [-node nodeURL] -tasklet tasklet_spec [-targ tasklets_args] : schedule job.");
	}
	/** node controlled by this xmlrpc server. */
	private Node node;
	/** Web server serving this xmlrpm server. */
	private WebServer webserver;
	// commands
	final static int CMD_HELP=0;
	final static int CMD_START=1;
	final static int CMD_STOP=2;
	final static int CMD_ADD=3;
	final static int CMD_REMOVE=4;
	final static int CMD_STATUS=5;
	final static int CMD_SUBMIT=6;
	final static int DEFAULT_HTTP_PORT=80;
	final static int DEFAULT_MAXLOAD=64;
}
