/*  $Id: XMLrpcRemoteNode.java 238 2006-04-30 20:07:59Z sdevaux $
 *
 * grant - light computationnal grid powered by ants
 * Tabs = 4 chars
 * copyright (C) 2004 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * Use of this software with non-GPL software requires specific licencing.
 * contact the author when such usage occures.
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.grant.xmlrpc;

import java.util.Hashtable;
import java.util.Vector;

import net.eduvax.grant.core.ExecReport;
import net.eduvax.grant.core.ExecRequest;
import net.eduvax.grant.core.Node;
import net.eduvax.grant.core.RemoteNode;
import net.eduvax.grant.core.TaskletEvent;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

/**
 * Node request broker on xml rpc.
 * sends node request thrue XML RPC protocole.
 * @version 
 * $Id: XMLrpcRemoteNode.java 238 2006-04-30 20:07:59Z sdevaux $
 * @author Sebastien Devaux
 */
class XMLrpcRemoteNode extends RemoteNode {
	/** 
	 * xml rpc call to route request to a node.
     * @param nd target node description.
     * @param er execute request.
	 * @return full routing path.
     */
	public ExecRequest submitRequest(ExecRequest er) {
		ExecRequest result=null;
		try {
			// Build our parameter list.
			Vector params = new Vector();
			addExecRequestToParams(params,er);
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.submitRequest", params);
			result = new ExecRequest(er.getTaskId(),
						er.getTaskURL(),
						er.getArgs(),
						(String)xmlrpcResult.get("routerPath"),
						(String)xmlrpcResult.get("status"));
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
        }
		return result;
	}
	/**
	 * xml rpc call to report execution.
	 * @param er execute report.
	 * @return task execution completion code.
	 */
	public int execReport(ExecReport er) {
		int result=-1;
		try {
			// Build our parameter list.
			Vector params = new Vector();
			addExecRequestToParams(params,er.getRequest());
			params.addElement(new Integer(er.getCr()));
			params.addElement(new Long(er.getTime()));
			params.addElement(new Float(er.getNodeLoad()));
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.execReport", params);
			result = ((Integer)xmlrpcResult.get("rc")).intValue();
        }
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               Integer.toString(exception.code) + ": " +
                               exception.toString());
        }
		return result;
	}
	/** 
	 * xml rpc call to remove a node from remote node.
     */
	public void removeNode(String name) {
		try {
			// Build our parameter list.
			Vector params = new Vector();
			params.add(name);
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.removeNode", params);
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
			exception.printStackTrace();							   
        }
	}
	/** 
	 * xml rpc call to add a node to remote node.
     */
	public Node addNode(String name) {
		try {
			// Build our parameter list.
			Vector params = new Vector();
			params.add(name);
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.addNode", params);
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
			exception.printStackTrace();							   
        }
        return null;
	}
	/** 
	 * xml rpc call to ask remote node for handling event.
	 * @param eventType evnet type
	 * @param event tasklet event
     */
	public void handleEvent(int eventType,TaskletEvent event) {
		try {
			// Build our parameter list.
			Vector params = new Vector();
			params.addElement(new Integer(eventType));
			addExecRequestToParams(params,event.getExecRequest());
			params.addElement(event.getData().toString());
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.handleEvent", params);
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
			exception.printStackTrace();							   
        }
	}
	/** 
	 * xml rpc call to get remote node status.
	 * @return status information.
     */
	public String status() {
		String result=null;
		try {
			// Build our parameter list.
			Vector params = new Vector();
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.status", params);
			result=(String)xmlrpcResult.get("status");
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
        }
		return result;
	}
	/** 
	 * xml rpc call to shutdown remote node.
     */
	public void shutdown() {
		try {
			// Build our parameter list.
			Vector params = new Vector();
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.shutdown", params);
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
        }
	}
	/** 
	 * xml rpc call to get remaote node load.
	 * @return load.
     */
	public float load() {
		float result=-1;
		try {
			// Build our parameter list.
			Vector params = new Vector();
			// Call the server, and get our result.
			Hashtable xmlrpcResult =
					(Hashtable) remoteNode.execute("grant.load", params);
			result=((Float)xmlrpcResult.get("load")).floatValue();
        } 
		catch (java.io.IOException ex) {
			ex.printStackTrace();
		}
		catch (XmlRpcException exception) {
            System.err.println("JavaClient: XML-RPC Fault #" +
                               exception.code + ": " +
                               exception.toString());
        }
		return result;
	}
	/**
	 * Create new xmlrpc node.
	 */
	public XMLrpcRemoteNode(String n) {
		super(n);
		try {
            String nodeName=getName();
            // remove xmlrpc+ from url.
            int index=nodeName.indexOf("+")+1;
            String remoteService=nodeName.substring(index);
			remoteNode = new XmlRpcClient(remoteService);
		}	
		catch (java.net.MalformedURLException ex) {
			ex.printStackTrace();
		}
	}
    // ------- Other Node methods implementation
    public String[] getKnownNodes() {
        // TODO complete implementation.
        return null;    
    }        
    //----------------------------------------------------------
    // PRIVATE METHODS
	/**
	 *
	 */ 
	private void addExecRequestToParams(Vector params,ExecRequest er) {
		params.addElement(er.getTaskId());
		params.addElement(er.getTaskURL());
		params.addElement(er.getArgs());
		params.addElement(er.getRouterPath());
		params.addElement(""+er.getRouteStatus());
	}	
    //----------------------------------------------------------
    // PRIVATE PROPERTIES
	/**
	 * remote xmlrpc node client connection.
	 */
	private XmlRpcClient remoteNode; 
}
